@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008-2010 Wind River Systems; see
+ * Copyright (c) 2008-2010,2012 Wind River Systems; see
  * guts/COPYRIGHT for information.
  *
  * static int
@@ -7,7 +7,7 @@
  *	int rc = -1;
  */
 
-	rc = wrap_openat(AT_FDCWD, path, flags, mode | O_LARGEFILE);
+	rc = wrap_openat(AT_FDCWD, path, flags | O_LARGEFILE, mode);
 
 /*	return rc;
  * }
