@@ -6,11 +6,14 @@
  * wrap_mkdirat(int dirfd, const char *path, mode_t mode) {
  *	int rc = -1;
  */
+	/* mask out mode bits appropriately */
+	mode = mode & ~pseudo_umask;
 #ifdef PSEUDO_NO_REAL_AT_FUNCTIONS
 	if (dirfd != AT_FDCWD) {
 		errno = ENOSYS;
 		return -1;
 	}
+
 	rc = real_mkdir(path, PSEUDO_FS_MODE(mode, 1));
 #else
 	rc = real_mkdirat(dirfd, path, PSEUDO_FS_MODE(mode, 1));
@@ -25,9 +28,10 @@
 		stat_rc = base_fstatat(dirfd, path, &buf, AT_SYMLINK_NOFOLLOW);
 #endif
 		if (stat_rc != -1) {
+			buf.st_mode = PSEUDO_DB_MODE(buf.st_mode, mode);
 			pseudo_client_op(OP_MKDIR, 0, -1, dirfd, path, &buf);
 		} else {
-			pseudo_debug(1, "mkdir of %s succeeded, but stat failed: %s\n",
+			pseudo_debug(PDBGF_OP, "mkdir of %s succeeded, but stat failed: %s\n",
 				path, strerror(errno));
 		}
 	}
